import { BehaviorSubject, combineLatest, Subject, Subscription } from 'rxjs';
import { Injectable, Injector, NgModule, Input } from '@angular/core';
import { map, filter } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * The Adaptation Controller is the core component of an adaptive UI.
 * It receives context information from Context Providers and evaluates Adaptation Rules via a Rule Engine session
 * to trigger adaptations on Adaptive Widgets in the UI.
 */
class AdaptationController {
    /**
     * Constructor: Inject Angulars Dependency Injector
     * @param {?} injector
     */
    constructor(injector) {
        /**
         * Keeps track of registered context provider components
         */
        this.contextProviders = [];
        /**
         * Keeps track of registered rule provider components
         */
        this.ruleProviders = [];
        /**
         * Observable stream of the context of use
         */
        this.context$ = new BehaviorSubject([]);
        /**
         * Observable stream of adaptation rules
         */
        this.rules$ = new BehaviorSubject([]);
        /**
         * Observable stream of Adaptation Actions
         */
        this.adaptationActions$ = new Subject();
        /**
         * Keeps track of subscriptions to observables
         */
        this.cpSubscription = new Subscription();
        this.rpSubscription = new Subscription();
        this.injector = injector;
    }
    /**
     * Register a Rule Evaluator component
     * @param {?} ruleEvaluatorToken
     * @return {?}
     */
    registerRuleEvaluator(ruleEvaluatorToken) {
        this.ruleEvaluator = this.injector.get(ruleEvaluatorToken);
    }
    /**
     * Registers a Context Provider to monitor context information.
     * Update set of context parameters when a change occurs.
     * @param {?} contextProviderToken
     * @return {?}
     */
    registerContextProvider(contextProviderToken) {
        // Inject and initialize Context Provider component as a Angular Service
        /** @type {?} */
        const contextProvider = this.injector.get(contextProviderToken);
        // add new component to list of Context Providers
        this.contextProviders.push(contextProvider);
        // get array of observables from all Context Providers
        /** @type {?} */
        const contextProviders = this.contextProviders.map((/**
         * @param {?} cp
         * @return {?}
         */
        cp => cp.getObservable()));
        // merge context parameters from all Context Providers
        /** @type {?} */
        const mergedContext$ = combineLatest(...contextProviders).pipe(map((/**
         * @param {?} nestedParams
         * @return {?}
         */
        nestedParams => [].concat(...nestedParams))));
        // observe changes to merged context and trigger evaluation process
        this.cpSubscription.unsubscribe();
        this.cpSubscription = mergedContext$.subscribe((/**
         * @param {?} params
         * @return {?}
         */
        params => this.updateContext(params)));
    }
    /**
     * Register a Rule Provider component
     * Update set of rules when a change occurs
     * @param {?} ruleProviderToken
     * @return {?}
     */
    registerRuleProvider(ruleProviderToken) {
        // Inject and initialize Rule Provider component as a Angular Service
        /** @type {?} */
        const ruleProvider = this.injector.get(ruleProviderToken);
        // add new component to list of Rule Providers
        this.ruleProviders.push(ruleProvider);
        // get array of observables from all Rule Providers
        /** @type {?} */
        const ruleProviders = this.ruleProviders.map((/**
         * @param {?} rp
         * @return {?}
         */
        rp => rp.getObservable()));
        // merge context parameters from all Rule Providers
        /** @type {?} */
        const mergedRules$ = combineLatest(...ruleProviders).pipe(map((/**
         * @param {?} nestedParams
         * @return {?}
         */
        nestedParams => [].concat(...nestedParams))));
        // observe changes to merged rules and trigger evaluation process
        this.rpSubscription.unsubscribe();
        this.rpSubscription = mergedRules$.subscribe((/**
         * @param {?} rules
         * @return {?}
         */
        rules => this.updateRules(rules)));
    }
    /**
     * Update context parameters and trigger evaluation process
     * @private
     * @param {?} contextParams
     * @return {?}
     */
    updateContext(contextParams) {
        console.debug("AC: Context updated", contextParams);
        this.context$.next(contextParams);
        this.evaluateAdaptationRules();
    }
    /**
     * Update adaptation rules and trigger evaluation process
     * @private
     * @param {?} rules
     * @return {?}
     */
    updateRules(rules) {
        console.debug("AC: Rules updated", rules);
        this.rules$.next(rules);
        this.evaluateAdaptationRules();
    }
    /**
     * Start evaluation process, if a Rule Evaluator is specified
     * @private
     * @return {?}
     */
    evaluateAdaptationRules() {
        if (this.ruleEvaluator)
            this.ruleEvaluator.evaluate(this.getRules(), this.getContext(), (/**
             * @param {?} action
             * @return {?}
             */
            (action) => this.fireAdaptation(action)));
        else
            console.log("AUI: Evaluation could not be started. No RuleEvaluator set.");
    }
    /**
     * Fire an Adaptation Action
     * Notify observers (Adaptive Widgets) about an Adaptation Action
     * @private
     * @param {?} action
     * @return {?}
     */
    fireAdaptation(action) {
        this.adaptationActions$.next(action);
        console.debug("Adaptation Controller: AdaptationAction triggered", action);
    }
    /**
     * An observable stream of Adaptation Actions
     * @return {?}
     */
    get AdaptationActions() {
        return this.adaptationActions$.asObservable();
    }
    /**
     * Returns current context parameters
     * @return {?}
     */
    getContext() {
        return this.context$.getValue();
    }
    /**
     * An observable stream of context paramteres
     * @return {?}
     */
    get Context() {
        return this.context$.asObservable();
    }
    /**
     * Returns current set adaptation rules
     * @return {?}
     */
    getRules() {
        return this.rules$.getValue();
    }
}
AdaptationController.decorators = [
    { type: Injectable }
];
/** @nocollapse */
AdaptationController.ctorParameters = () => [
    { type: Injector }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CoBAUIModule {
}
CoBAUIModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [],
                exports: [],
                providers: [AdaptationController]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * This component type is used to evaluate Adaptation Rules based on Context Parameters.
 * It triggers Adaptation Actions (defined in rules), if a rules condition is satisfied.
 * @abstract
 */
class RuleEvaluator {
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * This component type is used to obtain, manage and provide Adaptation Rules.
 * @abstract
 */
class RuleProvider {
    /**
     * Constructor: Initialize empty set of rules
     * @protected
     */
    constructor() {
        this.rules = [];
        this.rules$ = new BehaviorSubject([]);
    }
    /**
     * Publish rules to observers
     * @protected
     * @return {?}
     */
    updateRules() {
        this.rules$.next(this.rules);
    }
    /**
     * Add a rule to the set of rules and notify observers
     * @param {?} rule
     * @return {?}
     */
    addRule(rule) {
        this.rules.push(rule);
        this.rules$.next(this.rules);
    }
    /**
     * Remove a rule with given name and notify observers
     * @param {?} name
     * @return {?}
     */
    removeRule(name) {
        this.rules = this.rules.filter((/**
         * @param {?} rule
         * @return {?}
         */
        rule => rule.name !== name));
        this.rules$.next(this.rules);
    }
    /**
     * Return observable stream of rules that can be subscribed to by observers
     * @return {?}
     */
    getObservable() {
        return this.rules$.asObservable();
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const UpdateMethod = {
    EVENT_BASED: 'event-based',
    INTERVAL: 'interval',
};
/**
 * Base Class for "Context Provider" components.
 * Context Providers obtain information about the context of use (user, platform, environment) and report changes.
 * Provides base structure and functionality
 * @abstract
 */
class ContextProvider {
    /**
     * Constructor injects an Adaptation Controller component and receives configuration options
     * If updateMethod is chosen to be interval based, setup update interval
     * @protected
     * @param {?} method
     * @param {?=} time
     */
    constructor(method, time = 10000) {
        /**
         * Holds array of context parameters
         */
        this.contextParams = [];
        /**
         * An observable stream, that can be subscribed to
         */
        this.contextParams$ = new BehaviorSubject([]);
        // setup update interval
        if (method == UpdateMethod.INTERVAL) {
            /** @type {?} */
            const updateInterval = (/**
             * @return {?}
             */
            () => {
                this.updateContext();
            });
            setInterval(updateInterval, time);
        }
    }
    /**
     * Register a new context parameter to this Context Provider
     * @protected
     * @param {?} newParam
     * @return {?}
     */
    addContextParam(newParam) {
        if (this.contextParams.filter((/**
         * @param {?} param
         * @return {?}
         */
        param => param.key === newParam.key))) {
            this.contextParams.push(newParam);
        }
    }
    /**
     * Remove context parameter with given key from this Context Provider
     * @protected
     * @param {?} key
     * @return {?}
     */
    removeContextParam(key) {
        this.contextParams = this.contextParams.filter((/**
         * @param {?} param
         * @return {?}
         */
        param => param.key !== key));
    }
    /**
     * Returns a context parameter with given key
     * @protected
     * @param {?} key
     * @return {?}
     */
    getContextParam(key) {
        return this.contextParams.find((/**
         * @param {?} param
         * @return {?}
         */
        param => param.key == key));
    }
    /**
     * Modifies the value of a context parameter with given key
     * Notice: Does not implicitly report context update to observers.
     * @protected
     * @param {?} key
     * @param {?} newValue
     * @return {?}
     */
    modifyContextParam(key, newValue) {
        /** @type {?} */
        let oldParam = this.getContextParam(key);
        if (oldParam) {
            this.removeContextParam(key);
            this.addContextParam({ contextOfUse: oldParam.contextOfUse, type: oldParam.type, key: oldParam.key, value: newValue });
        }
    }
    /**
     * Return observable stream of context information
     * @return {?}
     */
    getObservable() {
        return this.contextParams$.asObservable();
    }
    /**
     * Update and emit context information to observers
     * @protected
     * @return {?}
     */
    updateContext() {
        this.contextParams$.next(this.contextParams);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * An AdaptiveWidget is a UI Component extended by adaptive functionality.
 * It defines and implements adaptations for itself, encapsulating the adaptation logic to the outside.
 * Adaptations can take various effects on the Widget, like changing its layout, modality or behaviour.
 * @abstract
 */
class AdaptiveWidget {
    /**
     * The Adaptation Controller is injected to enable and control dynamic context adaptation at runtime.
     * @protected
     * @param {?} adaptationController
     * @param {?} name
     */
    constructor(adaptationController, name) {
        this.adaptationController = adaptationController;
        this.name = name;
        /**
         * The namespace specifies a scope in which the widget adaptations are triggered.
         * This allows for more control over applying rules to certain instances or groups of this widget.
         */
        this.namespace = '';
        this.setupAdaptivity();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
    }
    /**
     * Register to the Adaptation Controller and listen for Adaptation Actions.
     * Check if Adaptation Action is addressed to this widget and perform given adaptation.
     * @private
     * @return {?}
     */
    setupAdaptivity() {
        this.adaptationController.AdaptationActions
            .pipe(filter((/**
         * @param {?} action
         * @return {?}
         */
        action => action.target === this.name && (action.scope.length === 0 || action.scope.includes(this.namespace)))))
            .subscribe((/**
         * @param {?} action
         * @return {?}
         */
        action => this.adapt(action)));
    }
    /**
     * Here the adaptation capabilities of the widget are implemented.
     * This function is meant to be overridden by inheriting widgets.
     * @param {?} action
     * @return {?}
     */
    adapt(action) {
        console.debug(`Widget (${this.name}/${this.namespace}): Action '${action.name}' performed`, action);
    }
}
AdaptiveWidget.propDecorators = {
    namespace: [{ type: Input, args: ['aui-namespace',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { CoBAUIModule, AdaptationController, RuleEvaluator, RuleProvider, UpdateMethod, ContextProvider, AdaptiveWidget };

//# sourceMappingURL=cobaui.js.map