import { InjectionToken, Injector, Type } from '@angular/core';
import { Observable } from 'rxjs';
import { ContextParam } from '../context/context-param.interface';
import { ContextProvider } from '../context/context-provider.base';
import { AdaptationRule } from '../rules/adaptation-rule.interface';
import { AdaptationAction } from '../ui/adaptation-action.interface';
import { RuleProvider } from '../rules/rule-provider.base';
import { RuleEvaluator } from '../rules/rule-evaluator.base';
/**
 * The Adaptation Controller is the core component of an adaptive UI.
 * It receives context information from Context Providers and evaluates Adaptation Rules via a Rule Engine session
 * to trigger adaptations on Adaptive Widgets in the UI.
 */
export declare class AdaptationController {
    private injector;
    /** Keeps track of registered context provider components */
    private readonly contextProviders;
    /** Keeps track of registered rule provider components */
    private readonly ruleProviders;
    /** Observable stream of the context of use */
    private readonly context$;
    /** Observable stream of adaptation rules */
    private readonly rules$;
    /** Rule evaluator for evaluating rules */
    private ruleEvaluator;
    /** Observable stream of Adaptation Actions */
    private readonly adaptationActions$;
    /** Keeps track of subscriptions to observables */
    private cpSubscription;
    private rpSubscription;
    /**
     * Constructor: Inject Angulars Dependency Injector
     */
    constructor(injector: Injector);
    /**
     * Register a Rule Evaluator component
     * @param ruleEvaluatorToken
     */
    registerRuleEvaluator(ruleEvaluatorToken: Type<RuleEvaluator> | InjectionToken<RuleEvaluator>): void;
    /**
     * Registers a Context Provider to monitor context information.
     * Update set of context parameters when a change occurs.
     * @param contextProviderToken
     */
    registerContextProvider(contextProviderToken: Type<ContextProvider> | InjectionToken<ContextProvider>): void;
    /**
     * Register a Rule Provider component
     * Update set of rules when a change occurs
     * @param ruleProviderToken
     */
    registerRuleProvider(ruleProviderToken: Type<RuleProvider> | InjectionToken<RuleProvider>): void;
    /**
     * Update context parameters and trigger evaluation process
     * @param contextParams
     */
    private updateContext;
    /**
     * Update adaptation rules and trigger evaluation process
     * @param rules
     */
    private updateRules;
    /**
     * Start evaluation process, if a Rule Evaluator is specified
     */
    private evaluateAdaptationRules;
    /**
     * Fire an Adaptation Action
     * Notify observers (Adaptive Widgets) about an Adaptation Action
     * @param action
     */
    private fireAdaptation;
    /** An observable stream of Adaptation Actions */
    readonly AdaptationActions: Observable<AdaptationAction>;
    /** Returns current context parameters */
    getContext(): ContextParam[];
    /** An observable stream of context paramteres */
    readonly Context: Observable<ContextParam[]>;
    /** Returns current set adaptation rules */
    getRules(): AdaptationRule[];
}
